from dataclasses import dataclass, field
from decimal import Decimal
from typing import Optional

from xsdata.models.datatype import XmlDateTime

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class InvoiceVatDetail:
    class Meta:
        name = "InvoiceVatDetailType"

    mark: Optional[str] = field(
        default=None,
        metadata={
            "name": "Mark",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    is_cancelled: Optional[bool] = field(
        default=None,
        metadata={
            "name": "IsCancelled",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    issue_date: Optional[XmlDateTime] = field(
        default=None,
        metadata={
            "name": "IssueDate",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    vat301: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat301",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat302: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat302",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat303: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat303",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat304: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat304",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat305: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat305",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat306: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat306",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat331: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat331",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat332: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat332",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat333: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat333",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat334: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat334",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat335: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat335",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat336: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat336",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat361: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat361",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat362: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat362",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat363: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat363",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat364: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat364",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat365: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat365",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat366: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat366",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat381: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat381",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat382: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat382",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat383: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat383",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat384: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat384",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat385: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat385",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat386: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat386",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat342: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat342",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat345: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat345",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat348: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat348",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat349: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat349",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat310: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat310",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat402: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat402",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat407: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat407",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat411: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat411",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat423: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat423",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat422: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "Vat422",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat_unclassified361: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "VatUnclassified361",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat_unclassified381: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "VatUnclassified381",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.aade_book_invoice_type import AadeBookInvoice

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class InvoicesDoc:
    """
    Παραστατικό ΑΑΔΕ.
    """

    class Meta:
        namespace = "http://www.aade.gr/myDATA/invoice/v1.0"

    invoice: List[AadeBookInvoice] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "min_occurs": 1,
        },
    )



from decimal import Decimal
from typing import List, Optional

from mydata.models.generated.invoice_row_type import InvoiceRow
from mydata.models.generated.invoice_summary_type import InvoiceSummary
from mydata.utils.invoice_taxes_summarizer import InvoiceTaxesSummarizer


class InvoiceRowsSummarizer:
    def __init__(self, rows: Optional[List[InvoiceRow]]):
        self.rows = rows
        self.total_net_value = Decimal(0)
        self.total_vat_amount = Decimal(0)
        self.total_taxes = Decimal(0)
        self.tax_summarizer = InvoiceTaxesSummarizer()

    def summarize_rows(self) -> None:
        if not self.rows:
            return

        for row in self.rows:
            self.total_net_value += abs(row.net_value or Decimal(0))
            self.total_vat_amount += abs(row.vat_amount or Decimal(0))
            self.tax_summarizer.add_taxes_from_invoice_row(row)

    def save_totals(self, summary: InvoiceSummary) -> None:
        net_value = round(
            (summary.total_net_value or Decimal(0)) + self.total_net_value, 2
        )
        summary.total_net_value = net_value

        vat_amount = round(
            (summary.total_vat_amount or Decimal(0)) + self.total_vat_amount, 2
        )
        summary.total_vat_amount = vat_amount

        self.save_taxes(summary)

        summary.total_gross_value = self.get_total_gross_value()

    def save_taxes(self, summary: InvoiceSummary) -> None:
        self.tax_summarizer.save_taxes(summary)

    def get_total_gross_value(self) -> Decimal:
        return round(self.total_net_value + self.total_vat_amount, 2)

    def get_total_taxes(self) -> Decimal:
        return self.total_taxes



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.income_classification_type import (
    IncomeClassification,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/incomeClassificaton/v1.0"


@dataclass
class InvoicesIncomeClassificationDetail:
    """
    Attributes:
        line_number: Γραμμή Παραστατικού
        income_classification_detail_data: Λίστα Χαρακτηρισμών Εσόδων
    """

    class Meta:
        name = "InvoicesIncomeClassificationDetailType"

    line_number: Optional[int] = field(
        default=None,
        metadata={
            "name": "lineNumber",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            "required": True,
        },
    )
    income_classification_detail_data: List[IncomeClassification] = field(
        default_factory=list,
        metadata={
            "name": "incomeClassificationDetailData",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            "min_occurs": 1,
        },
    )



from enum import Enum


class VatCategory(int, Enum):
    # ΦΠΑ συντελεστής 24%
    VAT_1 = 1
    # ΦΠΑ συντελεστής 13%
    VAT_2 = 2
    # ΦΠΑ συντελεστής 6%
    VAT_3 = 3
    # ΦΠΑ συντελεστής 17%
    VAT_4 = 4
    # ΦΠΑ συντελεστής 9%
    VAT_5 = 5
    # ΦΠΑ συντελεστής 4%
    VAT_6 = 6
    # Άνευ Φ.Π.Α. 0%
    VAT_7 = 7
    # Εγγραφές χωρίς ΦΠΑ (πχ Μισθοδοσία, Αποσβέσεις)
    VAT_8 = 8
    # ΦΠΑ συντελεστής 3% (αρ.31 ν.5057/2023)
    VAT_9 = 9
    # ΦΠΑ συντελεστής 4% (αρ.31 ν.5057/2023)
    VAT_10 = 10



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.invoices_income_classification_detail_type import (
    InvoicesIncomeClassificationDetail,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/incomeClassificaton/v1.0"


@dataclass
class InvoiceIncomeClassification:
    """
    Attributes:
        invoice_mark: Μοναδικός Αριθμός Καταχώρησης Παραστατικού
        classification_mark: Αποδεικτικό Λήψης Χαρακτηρισμού Εσόδων.
            Συμπληρώνεται από την Υπηρεσία
        entity_vat_number: ΑΦΜ Οντότητας Αναφοράς
        transaction_mode: Αιτιολογία Συναλλαγής
        invoices_income_classification_details:
    """

    class Meta:
        name = "InvoiceIncomeClassificationType"

    invoice_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceMark",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            "required": True,
        },
    )
    classification_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "classificationMark",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        },
    )
    entity_vat_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "entityVatNumber",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        },
    )
    transaction_mode: Optional[int] = field(
        default=None,
        metadata={
            "name": "transactionMode",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 2,
        },
    )
    invoices_income_classification_details: List[
        InvoicesIncomeClassificationDetail
    ] = field(
        default_factory=list,
        metadata={
            "name": "invoicesIncomeClassificationDetails",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        },
    )



from dataclasses import dataclass, field
from decimal import Decimal
from typing import List, Optional

from mydata.models.generated.expenses_classification_type import (
    ExpensesClassification,
)
from mydata.models.generated.income_classification_type import (
    IncomeClassification,
)

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class InvoiceSummary:
    """
    Attributes:
        total_net_value: Σύνολο Καθαρής Αξίας
        total_vat_amount: Σύνολο ΦΠΑ
        total_withheld_amount: Σύνολο Παρ. Φόρων
        total_fees_amount: Σύνολο Τελών
        total_stamp_duty_amount: Σύνολο Χαρτοσήμου
        total_other_taxes_amount: Σύνολο Λοιπών Φόρων
        total_deductions_amount: Σύνολο Κρατήσεων
        total_gross_value: Συνολική Αξία
        income_classification: Λίστα Χαρακτηρισμών Εσόδων
        expenses_classification:
    """

    class Meta:
        name = "InvoiceSummaryType"

    total_net_value: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalNetValue",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_vat_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalVatAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_withheld_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalWithheldAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_fees_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalFeesAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_stamp_duty_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalStampDutyAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_other_taxes_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalOtherTaxesAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_deductions_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalDeductionsAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    total_gross_value: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "totalGrossValue",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    income_classification: List[IncomeClassification] = field(
        default_factory=list,
        metadata={
            "name": "incomeClassification",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    expenses_classification: List[ExpensesClassification] = field(
        default_factory=list,
        metadata={
            "name": "expensesClassification",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from decimal import Decimal
from typing import List, Optional

from mydata.models.generated.invoice_row_type import InvoiceRow
from mydata.models.generated.invoice_summary_type import InvoiceSummary
from mydata.models.generated.tax_totals_type import TaxTotals
from mydata.models.tax_type import TaxType


class InvoiceTaxesSummarizer:
    def __init__(self, taxes: Optional[List[TaxTotals]] = None):
        self.taxes = taxes
        self.total_withheld_amount: Decimal = Decimal(0)
        self.total_fees_amount: Decimal = Decimal(0)
        self.total_stamp_duty_amount: Decimal = Decimal(0)
        self.total_other_taxes_amount: Decimal = Decimal(0)
        self.total_deductions_amount: Decimal = Decimal(0)
        self.total_informational_tax_amount: Decimal = Decimal(0)

    def summarize_taxes(self) -> None:
        if not self.taxes:
            return

        for tax in self.taxes:
            self.add_taxes_from_tax_totals(tax)

    def add_taxes_from_invoice_row(self, row: InvoiceRow) -> None:
        self.total_withheld_amount += abs(row.withheld_amount or Decimal(0))
        self.total_fees_amount += abs(row.fees_amount or Decimal(0))
        self.total_stamp_duty_amount += abs(row.stamp_duty_amount or Decimal(0))
        self.total_other_taxes_amount += abs(row.other_taxes_amount or Decimal(0))
        self.total_deductions_amount += abs(row.deductions_amount or Decimal(0))

        withheld_category = row.withheld_percent_category
        if withheld_category and not withheld_category.affects_total_gross_value:
            self.total_informational_tax_amount += abs(
                row.withheld_amount or Decimal(0)
            )

    def add_taxes_from_tax_totals(self, tax: TaxTotals) -> None:
        amount = abs(tax.tax_amount or Decimal(0))

        match tax.tax_type:
            case TaxType.TYPE_1:
                self.total_withheld_amount += amount
            case TaxType.TYPE_2:
                self.total_fees_amount += amount
            case TaxType.TYPE_3:
                self.total_other_taxes_amount += amount
            case TaxType.TYPE_4:
                self.total_stamp_duty_amount += amount
            case TaxType.TYPE_5:
                self.total_deductions_amount += amount

    def save_taxes(self, summary: InvoiceSummary) -> None:
        summary.total_withheld_amount = round(
            (summary.total_withheld_amount or Decimal(0)) + self.total_withheld_amount,
            2,
        )
        summary.total_fees_amount = round(
            (summary.total_fees_amount or Decimal(0)) + self.total_fees_amount, 2
        )
        summary.total_stamp_duty_amount = round(
            (summary.total_stamp_duty_amount or Decimal(0))
            + self.total_stamp_duty_amount,
            2,
        )
        summary.total_other_taxes_amount = round(
            (summary.total_other_taxes_amount or Decimal(0))
            + self.total_other_taxes_amount,
            2,
        )
        summary.total_deductions_amount = round(
            (summary.total_deductions_amount or Decimal(0))
            + self.total_deductions_amount,
            2,
        )

    def get_total_taxes(self) -> Decimal:
        return (
            -self.total_withheld_amount
            - self.total_deductions_amount
            + self.total_informational_tax_amount
            + self.total_fees_amount
            + self.total_stamp_duty_amount
            + self.total_other_taxes_amount
        )

    def save_totals(self, summary: InvoiceSummary) -> None:
        self.save_taxes(summary)

        gross_value = round(
            (summary.total_gross_value or Decimal(0)) + self.get_total_taxes(), 2
        )
        summary.total_gross_value = gross_value



from mydata.models.generated.invoice_summary_type import InvoiceSummary
from mydata.utils.classifications_grouper import ClassificationsGrouper
from mydata.utils.invoice_rows_summarizer import InvoiceRowsSummarizer
from mydata.utils.invoice_taxes_summarizer import InvoiceTaxesSummarizer


class InvoiceSummarizer:
    def __init__(self, invoice):
        self.invoice = invoice

    def summarize(self) -> InvoiceSummary:
        summary = InvoiceSummary()

        rows_summarizer = InvoiceRowsSummarizer(rows=self.invoice.invoice_details)
        rows_summarizer.summarize_rows()
        rows_summarizer.save_totals(summary)

        taxes_summarizer = InvoiceTaxesSummarizer(taxes=self.invoice.taxes_totals.taxes)
        taxes_summarizer.summarize_taxes()
        taxes_summarizer.save_totals(summary)

        classifications_group = ClassificationsGrouper(
            rows=self.invoice.invoice_details
        )
        icls, ecls = classifications_group.create_groups()

        summary.income_classification = icls
        summary.expenses_classification = ecls

        return summary



from dataclasses import dataclass, field
from decimal import Decimal
from typing import List, Optional

from xsdata.models.datatype import XmlDate, XmlTime

from mydata.models.generated.currency_type import Currency
from mydata.models.generated.entity_type import Entity
from mydata.models.generated.invoice_type import InvoiceType
from mydata.models.generated.other_delivery_note_header_type import (
    OtherDeliveryNoteHeader,
)

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class InvoiceHeader:
    """
    Attributes:
        series: Σειρά Παραστατικού
        aa: ΑΑ Παραστατικού
        issue_date: Ημερομηνία Έκδοσης
        invoice_type: Είδος Παραστατικού
        vat_payment_suspension: Αναστολή Καταβολής ΦΠΑ
        currency: Νόμισμα
        exchange_rate: Ισοτιμία
        correlated_invoices: Συσχετιζόμενα Παραστατικά
        self_pricing: Ένδειξη Αυτοτιμολόγησης
        dispatch_date: Ημερομηνία  Έναρξης Αποστολής
        dispatch_time: Ώρα Έναρξης Αποστολής
        vehicle_number: Αριθμός Οχήματος
        move_purpose: Σκοπός Διακίνησης
        fuel_invoice: Παραστατικό Καυσίμων
        special_invoice_category: Ειδική Κατηγορία Παραστατικού
        invoice_variation_type: Τύπος Απόκλισης Παραστατικού
        other_correlated_entities: Λοιπές συσχετιζόμενες οντοτήτες
        other_delivery_note_header: Λοιπά Γενικά Στοιχεία Διακίνησης
        is_delivery_note: Ένδειξη Παραστατικού Διακίνησης
        other_move_purpose_title: Τίτλος της Λοιπής Αιτίας Διακίνησης
        third_party_collection: Ένδειξη Εισπράξης Τρίτων
        multiple_connected_marks: Πολλαπλά Συνδεόμενα MARKs
        table_aa: AA ΤΡΑΠΕΖΙOY (για Δελτία Παραγγελίας Εστίασης)
        total_cancel_delivery_orders: Ένδειξη συνολικής αναίρεσης
            Δελτίων Παραγελίας
    """

    class Meta:
        name = "InvoiceHeaderType"

    series: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 50,
        },
    )
    aa: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 50,
        },
    )
    issue_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "issueDate",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    invoice_type: Optional[InvoiceType] = field(
        default=None,
        metadata={
            "name": "invoiceType",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    vat_payment_suspension: Optional[bool] = field(
        default=None,
        metadata={
            "name": "vatPaymentSuspension",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    currency: Optional[Currency] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    exchange_rate: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "exchangeRate",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_exclusive": Decimal("0"),
            "max_inclusive": Decimal("50000"),
            "fraction_digits": 5,
        },
    )
    correlated_invoices: List[int] = field(
        default_factory=list,
        metadata={
            "name": "correlatedInvoices",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    self_pricing: Optional[bool] = field(
        default=None,
        metadata={
            "name": "selfPricing",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    dispatch_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "dispatchDate",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    dispatch_time: Optional[XmlTime] = field(
        default=None,
        metadata={
            "name": "dispatchTime",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vehicle_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "vehicleNumber",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 150,
        },
    )
    move_purpose: Optional[int] = field(
        default=None,
        metadata={
            "name": "movePurpose",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 19,
        },
    )
    fuel_invoice: Optional[bool] = field(
        default=None,
        metadata={
            "name": "fuelInvoice",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    special_invoice_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "specialInvoiceCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 12,
        },
    )
    invoice_variation_type: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceVariationType",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 4,
        },
    )
    other_correlated_entities: List[Entity] = field(
        default_factory=list,
        metadata={
            "name": "otherCorrelatedEntities",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    other_delivery_note_header: Optional[OtherDeliveryNoteHeader] = field(
        default=None,
        metadata={
            "name": "otherDeliveryNoteHeader",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    is_delivery_note: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isDeliveryNote",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    other_move_purpose_title: Optional[str] = field(
        default=None,
        metadata={
            "name": "otherMovePurposeTitle",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 150,
        },
    )
    third_party_collection: Optional[bool] = field(
        default=None,
        metadata={
            "name": "thirdPartyCollection",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    multiple_connected_marks: List[int] = field(
        default_factory=list,
        metadata={
            "name": "multipleConnectedMarks",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    table_aa: Optional[str] = field(
        default=None,
        metadata={
            "name": "tableAA",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 50,
        },
    )
    total_cancel_delivery_orders: Optional[bool] = field(
        default=None,
        metadata={
            "name": "totalCancelDeliveryOrders",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



import logging
from typing import Any, Type

from xsdata.formats.dataclass.context import XmlContext
from xsdata.formats.dataclass.parsers import XmlParser


class XMLResponseParser:
    NAMESPACE_MAP = {
        None: "http://www.aade.gr/myDATA/invoice/v1.0",
        "xsi": "http://www.w3.org/2001/XMLSchema-instance",
        "icls": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        "ecls": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
    }

    def __init__(self):
        self.parser = XmlParser(context=XmlContext())

    def parse(self, xml_data: str, model: Type[Any]) -> Any:
        try:
            return self.parser.from_string(xml_data, model, ns_map=self.NAMESPACE_MAP)
        except Exception as e:
            logging.error(f"Error during XML parsing {e}", exc_info=True)



"""HTTP client module responsible for low-level HTTP communication with the MyData API."""

from typing import Dict, Optional

import requests

from .exceptions import (
    MyDataAuthenticationException,
    MyDataConnectionException,
    MyDataException,
    MyDataRateLimitExceededException,
)


class HttpClient:
    """A thin wrapper around `requests` providing convenient get/post methods and common headers.

    Attributes:
        session (requests.Session): A requests session object that maintains connection pooling and header state.
    """

    def __init__(
        self, headers: Dict[str, str], session: Optional[requests.Session] = None
    ):
        """Initialize the HttpClient.

        Args:
            headers (Dict[str, str]): Default headers to include in all requests.
            session (Optional[requests.Session], optional): A custom requests session. Defaults to None.
        """
        self.session = session or requests.Session()
        self.session.headers.update(headers)

    def post(self, url: str, data: str, timeout: int = 30) -> requests.Response:
        """Perform a POST request.

        Args:
            url (str): The request URL.
            data (str): The request payload (XML).
            timeout (int, optional): Request timeout in seconds. Defaults to 30.

        Returns:
            requests.Response: The HTTP response.

        Raises:
            MyDataHTTPException: If the response status code indicates an error.
        """
        try:
            response = self.session.post(url, data=data, timeout=timeout)
            self._raise_for_status(response, url)
            return response
        except requests.RequestException as exc:
            raise MyDataConnectionException(status_code=0, url=url, message=str(exc))

    def get(
        self, url: str, params: Optional[Dict[str, str]] = None, timeout: int = 30
    ) -> requests.Response:
        """Perform a GET request.

        Args:
            url (str): The request URL.
            params (Optional[Dict[str, str]]): URL query parameters. Defaults to None.
            timeout (int, optional): Request timeout in seconds. Defaults to 30.

        Returns:
            requests.Response: The HTTP response.

        Raises:
            MyDataHTTPException: If the response status code indicates an error.
        """
        try:
            response = self.session.get(url, params=params, timeout=timeout)
            self._raise_for_status(response, url)
            return response
        except requests.RequestException as exc:
            raise MyDataConnectionException(status_code=0, url=url, message=str(exc))

    @staticmethod
    def _raise_for_status(response: requests.Response, url: str):
        """Raise appropriate exceptions based on HTTP response status code."""
        status_code = response.status_code

        if status_code == 401 or status_code == 403:
            raise MyDataAuthenticationException(
                status_code,
                url,
                "Authentication failed. Please check your user id and subscription key.",
            )
        elif status_code == 429:
            raise MyDataRateLimitExceededException(status_code, url, response.text)
        elif status_code == 0:
            raise MyDataConnectionException(status_code, url, response.text)
        elif not response.ok:
            raise MyDataException(status_code, url, response.text)



from enum import Enum


class WithheldPercentCategory(int, Enum):
    # Περιπτ. β’- Τόκοι - 15%
    TAX_1 = 1
    #  Περιπτ. γ’ - Δικαιώματα - 20%
    TAX_2 = 2
    #  Περιπτ. δ’ - Αμοιβές Συμβουλών Διοίκησης - 20%
    TAX_3 = 3
    #  Περιπτ. δ’ - Τεχνικά Έργα - 3%
    TAX_4 = 4
    #  Υγρά καύσιμα και προϊόντα καπνοβιομηχανίας 1%
    TAX_5 = 5
    #  Λοιπά Αγαθά 4%
    TAX_6 = 6
    #  Παροχή Υπηρεσιών 8%
    TAX_7 = 7
    #  Προκαταβλητέος Φόρος Αρχιτεκτόνων και Μηχανικών επί Συμβατικών Αμοιβών, για Εκπόνηση Μελετών και Σχεδίων 4%
    TAX_8 = 8
    #  Προκαταβλητέος Φόρος Αρχιτεκτόνων και Μηχανικών επί Συμβατικών Αμοιβών, που αφορούν οποιασδήποτε άλλης φύσης έργα 10%
    TAX_9 = 9
    #  Προκαταβλητέος Φόρος στις Αμοιβές Δικηγόρων 15%
    TAX_10 = 10
    #  Παρακράτηση Φόρου Μισθωτών Υπηρεσιών παρ. 1 αρ. 15 ν. 4172/2013 ποσό
    TAX_11 = 11
    #  Παρακράτηση Φόρου Μισθωτών Υπηρεσιών παρ. 2 αρ. 15 ν. 4172/2013 - Αξιωματικών Εμπορικού Ναυτικού 15%
    TAX_12 = 12
    #  Παρακράτηση Φόρου Μισθωτών Υπηρεσιών παρ. 2 αρ. 15 ν. 4172/2013 - Κατώτερο Πλήρωμα Εμπορικού Ναυτικού 10%
    TAX_13 = 13
    #  Παρακράτηση Ειδικής Εισφοράς Αλληλεγγύης ποσό
    TAX_14 = 14
    #  Παρακράτηση Φόρου Αποζημίωσης λόγω Διακοπής Σχέσης Εργασίας παρ. 3 αρ. 15 ν. 4172/2013 ποσό
    TAX_15 = 15
    #  Παρακρατήσεις συναλλαγών αλλοδαπής βάσει συμβάσεων αποφυγής διπλής φορολογίας (Σ.Α.Δ.Φ.) ποσό
    TAX_16 = 16
    #  Λοιπές Παρακρατήσεις Φόρου ποσό
    TAX_17 = 17
    #  Παρακράτηση Φόρου Μερίσματα περ.α παρ. 1 αρ. 64 ν. 4172/2013 5%
    TAX_18 = 18

    @property
    def affects_total_gross_value(self) -> bool:
        return self not in [
            WithheldPercentCategory.TAX_8,
            WithheldPercentCategory.TAX_9,
            WithheldPercentCategory.TAX_10,
        ]



from dataclasses import dataclass, field
from typing import Optional

from mydata.models.generated.address_type import Address
from mydata.models.generated.country_type import Country

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class Party:
    """
    Attributes:
        vat_number:
        country: Κωδ. Χώρας
        branch: Αρ. Εγκατάστασης
        name:
        address: Διεύθυνση
        document_id_no:
        supply_account_no: Αρ. Παροχής Ηλ. Ρεύματος
        country_document_id: Κωδ. Χώρας Έκδοσης Επίσημου Εγγράφου
    """

    class Meta:
        name = "PartyType"

    vat_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "vatNumber",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 30,
        },
    )
    country: Optional[Country] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    branch: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 200,
        },
    )
    address: Optional[Address] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    document_id_no: Optional[str] = field(
        default=None,
        metadata={
            "name": "documentIdNo",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 100,
        },
    )
    supply_account_no: Optional[str] = field(
        default=None,
        metadata={
            "name": "supplyAccountNo",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 100,
        },
    )
    country_document_id: Optional[Country] = field(
        default=None,
        metadata={
            "name": "countryDocumentId",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.invoice_expenses_classification_type import (
    InvoiceExpensesClassification,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/expensesClassificaton/v1.0"


@dataclass
class ExpensesClassificationsDoc:
    """
    Χαρατηρισμοί Εξόδων Πρότυπων Παραστατικών ΑΑΔΕ.
    """

    class Meta:
        namespace = "https://www.aade.gr/myDATA/expensesClassificaton/v1.0"

    expenses_invoice_classification: List[InvoiceExpensesClassification] = (
        field(
            default_factory=list,
            metadata={
                "name": "expensesInvoiceClassification",
                "type": "Element",
                "min_occurs": 1,
            },
        )
    )



from enum import Enum


class VatExemption(int, Enum):
    #  Χωρίς ΦΠΑ – άρθρο 2 και 3 του Κώδικα ΦΠΑ
    TYPE_1 = 1
    #  Χωρίς ΦΠΑ - άρθρο 5 του Κώδικα ΦΠΑ
    TYPE_2 = 2
    #  Χωρίς ΦΠΑ - άρθρο 13 του Κώδικα ΦΠΑ
    TYPE_3 = 3
    #  Χωρίς ΦΠΑ - άρθρο 14 του Κώδικα ΦΠΑ
    TYPE_4 = 4
    #  Χωρίς ΦΠΑ - άρθρο 16 του Κώδικα ΦΠΑ
    TYPE_5 = 5
    #  Χωρίς ΦΠΑ - άρθρο 19 του Κώδικα ΦΠΑ
    TYPE_6 = 6
    #  Χωρίς ΦΠΑ - άρθρο 22 του Κώδικα ΦΠΑ
    TYPE_7 = 7
    #  Χωρίς ΦΠΑ - άρθρο 24 του Κώδικα ΦΠΑ
    TYPE_8 = 8
    #  Χωρίς ΦΠΑ - άρθρο 25 του Κώδικα ΦΠΑ
    TYPE_9 = 9
    #  Χωρίς ΦΠΑ - άρθρο 26 του Κώδικα ΦΠΑ
    TYPE_10 = 10
    #  Χωρίς ΦΠΑ - άρθρο 27 του Κώδικα ΦΠΑ
    TYPE_11 = 11
    #  Χωρίς ΦΠΑ - άρθρο 27 - Πλοία Ανοικτής Θαλάσσης του Κώδικα ΦΠΑ
    TYPE_12 = 12
    #  Χωρίς ΦΠΑ - άρθρο 27.1.γ - Πλοία Ανοικτής Θαλάσσης του Κώδικα ΦΠΑ
    TYPE_13 = 13
    #  Χωρίς ΦΠΑ - άρθρο 28 του Κώδικα ΦΠΑ
    TYPE_14 = 14
    #  Χωρίς ΦΠΑ - άρθρο 39 του Κώδικα ΦΠΑ
    TYPE_15 = 15
    #  Χωρίς ΦΠΑ - άρθρο 39α του Κώδικα ΦΠΑ
    TYPE_16 = 16
    #  Χωρίς ΦΠΑ - άρθρο 40 του Κώδικα ΦΠΑ
    TYPE_17 = 17
    #  Χωρίς ΦΠΑ - άρθρο 41 του Κώδικα ΦΠΑ
    TYPE_18 = 18
    #  Χωρίς ΦΠΑ - άρθρο 47 του Κώδικα ΦΠΑ
    TYPE_19 = 19
    #  ΦΠΑ εμπεριεχόμενος - άρθρο 43 του Κώδικα ΦΠΑ
    TYPE_20 = 20
    #  ΦΠΑ εμπεριεχόμενος - άρθρο 44 του Κώδικα ΦΠΑ
    TYPE_21 = 21
    #  ΦΠΑ εμπεριεχόμενος - άρθρο 45 του Κώδικα ΦΠΑ
    TYPE_22 = 22
    #  ΦΠΑ εμπεριεχόμενος - άρθρο 46 του Κώδικα ΦΠΑ
    TYPE_23 = 23
    #  Χωρίς ΦΠΑ - άρθρο 6 του Κώδικα ΦΠΑ
    TYPE_24 = 24
    #  Χωρίς ΦΠΑ - ΠΟΛ.1029/1995
    TYPE_25 = 25
    #  Χωρίς ΦΠΑ - ΠΟΛ.1167/2015
    TYPE_26 = 26
    #  Λοιπές Εξαιρέσεις ΦΠΑ
    TYPE_27 = 27
    # Χωρίς ΦΠΑ – άρθρο 24 περ. β' παρ.1 του Κώδικα ΦΠΑ, (Tax Free)
    TYPE_28 = 28
    # Χωρίς ΦΠΑ – άρθρο 47β, του Κώδικα ΦΠΑ (OSS μη ενωσιακό καθεστώς)
    TYPE_29 = 29
    # Χωρίς ΦΠΑ – άρθρο 47γ, του Κώδικα ΦΠΑ (OSS ενωσιακό καθεστώς)
    TYPE_30 = 30
    # Χωρίς ΦΠΑ – άρθρο 47δ του Κώδικα ΦΠΑ (IOSS)
    TYPE_31 = 31



"""Main client class for interacting with the AADE/myData API."""

from typing import Any, Dict, List, Optional, Type

from .client_config import MyDataClientConfig
from .exceptions import MyDataXMLParseException
from .http_client import HttpClient
from .models.generated.requested_doc import RequestedDoc
from .models.generated.response_doc import ResponseDoc
from .models.invoice import Invoice
from .models.invoices_document import InvoicesDocument
from .utils.xml_parser import XMLResponseParser
from .utils.xml_serializer import XmlSerializerService


class MyDataClient:
    """Client class for interacting with AADE/myData API endpoints.

    Provides methods to send invoices and request documents. Serializes requests
    to XML and deserializes responses to Python objects.
    """

    def __init__(
        self,
        user_id: str,
        subscription_key: str,
        config: Optional[MyDataClientConfig] = None,
        http_client: Optional[HttpClient] = None,
        serializer: Optional[XmlSerializerService] = None,
        deserializer: Optional[XMLResponseParser] = None,
    ):
        """Initialize the MyDataClient.

        Args:
            user_id (str): The AADE user ID.
            subscription_key (str): The subscription key provided by AADE.
            config (Optional[MyDataClientConfig], optional): Configuration object. Defaults to None.
            http_client (Optional[HttpClient], optional): A pre-configured HttpClient. Defaults to None.
            serializer (Optional[XmlSerializerService], optional): XML serializer. Defaults to None.
            deserializer (Optional[XMLResponseParser], optional): XML response parser. Defaults to None.
        """
        self.user_id = user_id
        self.subscription_key = subscription_key
        self.config = config or MyDataClientConfig()
        self.serializer = serializer or XmlSerializerService()
        self.deserializer = deserializer or XMLResponseParser()

        self.http_client = http_client or HttpClient(
            headers=self._default_headers(), session=self.config.session
        )

    def _default_headers(self) -> dict:
        """Default headers for all API requests."""
        return {
            "aade-user-id": self.user_id,
            "ocp-apim-subscription-key": self.subscription_key,
            "Content-Type": "text/xml",
        }

    def _get(self, endpoint: str, params: Dict[str, str] = None) -> str:
        """Helper method to perform GET requests.

        Args:
            endpoint (str): The endpoint path (relative to base_url).
            params (Dict[str, str], optional): URL query parameters. Defaults to None.

        Returns:
            str: The response text.

        Raises:
            MyDataHTTPException: If an HTTP error occurs.
        """
        url = f"{self.config.base_url}{endpoint}"
        response = self.http_client.get(url, params=params, timeout=self.config.timeout)
        return response.text

    def _post(self, endpoint: str, data: str) -> str:
        """Helper method to perform POST requests.

        Args:
            endpoint (str): The endpoint path (relative to base_url).
            data (str): The XML payload.

        Returns:
            str: The response text.

        Raises:
            MyDataHTTPException: If an HTTP error occurs.
        """
        url = f"{self.config.base_url}{endpoint}"
        response = self.http_client.post(url, data=data, timeout=self.config.timeout)
        return response.text

    def _parse_response(self, xml_data: str, model: Type[Any]) -> Any:
        """Parse XML response data into the specified model.

        Args:
            xml_data (str): The raw XML response.
            model (Type[Any]): The model class to deserialize into.

        Returns:
            Any: An instance of the model populated with parsed data.

        Raises:
            MyDataXMLParseException: If XML parsing fails.
        """
        try:
            return self.deserializer.parse(xml_data, model)
        except Exception as e:
            raise MyDataXMLParseException(e)

    def send_invoice(
        self, invoice: Invoice, response_model: Type[Any] = ResponseDoc
    ) -> ResponseDoc:
        """Send a single invoice to AADE/myData.

        Args:
            invoice (Invoice): The invoice object to be sent.
            response_model (Type[Any], optional): The model to parse the response into. Defaults to ResponseDoc.

        Returns:
            Any: Parsed response object.
        """
        return self.send_invoices([invoice], response_model=response_model)

    def send_invoices(
        self,
        invoices: List[Invoice],
        response_model: Type[Any] = ResponseDoc,
    ) -> ResponseDoc:
        """Send multiple invoices to AADE/myData.

        Args:
            invoices (List[Invoice]): A list of Invoice objects.
            response_model (Type[Any], optional): The model to parse the response into. Defaults to ResponseDoc.

        Returns:
            ResponseDoc: Parsed response object.
        """
        doc = InvoicesDocument(invoices=invoices, serializer=self.serializer)
        xml_data = doc.as_xml()
        response = self._post("SendInvoices", xml_data)
        return self._parse_response(response, response_model)

    def request_transmitted_docs(
        self,
        mark: int = 0,
        response_model: Type[Any] = RequestedDoc,
    ) -> RequestedDoc:
        """Request transmitted documents from AADE/myData by mark The API will return all documents with mark greater
        than or equal to the provided mark.

        Args:
            mark (int): The unique document mark.
            response_model (Type[Any], optional): The model to parse the response into. Defaults to RequestedDoc.

        Returns:
            RequestedDoc: Parsed requested document model.
        """
        response = self._get("RequestTransmittedDocs", params={"mark": str(mark)})
        return self._parse_response(response, response_model)

    def cancel_invoice(self, mark: int, response_model: Type[Any] = ResponseDoc):
        """Cancels an invoice by its mark

        Args:
            mark (int): The unique document mark to be canceled

        Returns:
            ResponseDoc: Parsed response object.
        """
        response = self._post(f"CancelInvoice?mark={mark}", None)
        return self._parse_response(response, response_model)



from dataclasses import dataclass, field
from typing import Optional

from mydata.models.generated.party_type import Party

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class Entity:
    """
    Attributes:
        type_value: Κατηγορία
        entity_data: Στοιχεία Οντότητας
    """

    class Meta:
        name = "EntityType"

    type_value: Optional[int] = field(
        default=None,
        metadata={
            "name": "type",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": 1,
            "max_inclusive": 6,
        },
    )
    entity_data: Optional[Party] = field(
        default=None,
        metadata={
            "name": "entityData",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )



from typing import List, Optional

from xsdata.formats.dataclass.models.elements import XmlType

from mydata.models.generated.invoices_doc import InvoicesDoc
from mydata.models.invoice import Invoice
from mydata.utils.xml_serializer import XmlSerializerService


class InvoicesDocument(InvoicesDoc):
    NAMESPACE_MAP = {
        None: "http://www.aade.gr/myDATA/invoice/v1.0",
        "xsi": "http://www.w3.org/2001/XMLSchema-instance",
        "icls": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        "ecls": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
    }

    def __init__(self, invoices=List[Invoice], serializer=None):
        self.invoice = invoices
        self.serializer = serializer or XmlSerializerService()

    class Meta:
        name = "InvoicesDoc"
        nillable = False
        namespace = "http://www.aade.gr/myDATA/invoice/v1.0"
        type = XmlType.ELEMENT

    def as_xml(self, ns_map: Optional[dict] = None) -> str:
        return self.serializer.serialize(
            self, ns_map=ns_map or self.NAMESPACE_MAP
        )



from typing import Dict, Optional

from xsdata.formats.dataclass.context import XmlContext
from xsdata.formats.dataclass.serializers import XmlSerializer
from xsdata.formats.dataclass.serializers.config import SerializerConfig


class XmlSerializerService:
    def __init__(self):
        config = SerializerConfig(indent="  ")
        self.serializer = XmlSerializer(context=XmlContext(), config=config)

    def serialize(self, obj: object, ns_map: Optional[Dict[str, str]] = None) -> str:
        return self.serializer.render(obj, ns_map=ns_map)



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.continuation_token_type_2 import (
    ContinuationTokenType2,
)
from mydata.models.generated.invoice_provider_type import InvoiceProvider


@dataclass
class RequestedProviderDoc:
    """
    Παραστατικά από Πάροχο.
    """

    continuation_token: List[ContinuationTokenType2] = field(
        default_factory=list,
        metadata={
            "name": "continuationToken",
            "type": "Element",
            "sequence": 1,
        },
    )
    invoice_provider_type: List[InvoiceProvider] = field(
        default_factory=list,
        metadata={
            "name": "InvoiceProviderType",
            "type": "Element",
            "sequence": 1,
        },
    )



from dataclasses import dataclass, field
from typing import Optional

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class ProviderSignature:
    """
    Attributes:
        signing_author: Provider’s Id
        signature: Υπογραφή
    """

    class Meta:
        name = "ProviderSignatureType"

    signing_author: Optional[str] = field(
        default=None,
        metadata={
            "name": "SigningAuthor",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 20,
        },
    )
    signature: Optional[str] = field(
        default=None,
        metadata={
            "name": "Signature",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )



from dataclasses import dataclass, field
from typing import Optional

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class Ecrtoken:
    """
    Attributes:
        signing_author: ECR id: Αριθμός μητρώου του φορολογικού
            μηχανισμού
        session_number: Μοναδικός 6-ψήφιος κωδικός που χαρακτηρίζει την
            κάθε συναλλαγή
    """

    class Meta:
        name = "ECRTokenType"

    signing_author: Optional[str] = field(
        default=None,
        metadata={
            "name": "SigningAuthor",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 15,
        },
    )
    session_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "SessionNumber",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "length": 6,
        },
    )



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.invoice_header_type import InvoiceHeader
from mydata.models.generated.invoice_row_type import InvoiceRow
from mydata.models.generated.invoice_summary_type import InvoiceSummary
from mydata.models.generated.party_type import Party
from mydata.models.generated.payment_method_detail_type import (
    PaymentMethodDetail,
)
from mydata.models.generated.tax_totals_type import TaxTotals
from mydata.models.generated.transport_detail_type import TransportDetail

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class AadeBookInvoice:
    """
    Attributes:
        uid: Αναγνωριστικό Παραστατικού
        mark: Μοναδικός Αριθμός Καταχώρησης Παραστατικού
        cancelled_by_mark: Μοναδικός Αριθμός Καταχώρησης Ακυρωτικού
        authentication_code: Συμβολοσειρά Αυθεντικοποίησης Παρόχου
        transmission_failure: Αδυναμία Επικοινωνίας Παρόχου ή Αδυναμία
            διαβίβασης ERP
        issuer: Στοιχεία Εκδότη
        counterpart: Στοιχεία Λήπτη
        invoice_header: Γενικά Στοιχεία
        payment_methods: Πληρωμές
        invoice_details: Λεπτομέρειες Παραστατικού
        taxes_totals: Σύνολα Φόρων
        invoice_summary: Συγκεντρωτικά Στοιχεία
        qr_code_url: QR Code Url
        other_transport_details: Λοιπές Λεπτομέρειες Διακίνησης (Ορισμός
            - Αλλαγή Μτφ Μέσων, Μεταφορτώσεις, κλπ)
    """

    class Meta:
        name = "AadeBookInvoiceType"

    uid: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    mark: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    cancelled_by_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "cancelledByMark",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    authentication_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "authenticationCode",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    transmission_failure: Optional[int] = field(
        default=None,
        metadata={
            "name": "transmissionFailure",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 3,
        },
    )
    issuer: Optional[Party] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    counterpart: Optional[Party] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    invoice_header: Optional[InvoiceHeader] = field(
        default=None,
        metadata={
            "name": "invoiceHeader",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    payment_methods: Optional["AadeBookInvoice.PaymentMethods"] = field(
        default=None,
        metadata={
            "name": "paymentMethods",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    invoice_details: List[InvoiceRow] = field(
        default_factory=list,
        metadata={
            "name": "invoiceDetails",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_occurs": 1,
        },
    )
    taxes_totals: Optional["AadeBookInvoice.TaxesTotals"] = field(
        default=None,
        metadata={
            "name": "taxesTotals",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    invoice_summary: Optional[InvoiceSummary] = field(
        default=None,
        metadata={
            "name": "invoiceSummary",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    qr_code_url: Optional[str] = field(
        default=None,
        metadata={
            "name": "qrCodeUrl",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    other_transport_details: List[TransportDetail] = field(
        default_factory=list,
        metadata={
            "name": "otherTransportDetails",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )

    @dataclass
    class PaymentMethods:
        """
        Attributes:
            payment_method_details: Στοιχεία Πληρωμών
        """

        payment_method_details: List[PaymentMethodDetail] = field(
            default_factory=list,
            metadata={
                "name": "paymentMethodDetails",
                "type": "Element",
                "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
                "min_occurs": 1,
            },
        )

    @dataclass
    class TaxesTotals:
        taxes: List[TaxTotals] = field(
            default_factory=list,
            metadata={
                "type": "Element",
                "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
                "min_occurs": 1,
            },
        )



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.invoice_income_classification_type import (
    InvoiceIncomeClassification,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/incomeClassificaton/v1.0"


@dataclass
class IncomeClassificationsDoc:
    """
    Χαρατηρισμοί Εσόδων Πρότυπων Παραστατικών ΑΑΔΕ.
    """

    class Meta:
        namespace = "https://www.aade.gr/myDATA/incomeClassificaton/v1.0"

    income_invoice_classification: List[InvoiceIncomeClassification] = field(
        default_factory=list,
        metadata={
            "name": "incomeInvoiceClassification",
            "type": "Element",
            "min_occurs": 1,
        },
    )



from dataclasses import dataclass, field
from typing import Optional

from mydata.models.generated.address_type import Address

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class OtherDeliveryNoteHeader:
    """
    Attributes:
        loading_address: Διεύθυνση Φόρτωσης
        delivery_address: Διεύθυνση Παράδοσης
        start_shipping_branch: Εγκατάσταση έναρξης διακίνησης (Εκδότη)
        complete_shipping_branch: Εγκατάσταση ολοκλήρωσης διακίνησης
            (Λήπτη)
    """

    class Meta:
        name = "OtherDeliveryNoteHeaderType"

    loading_address: Optional[Address] = field(
        default=None,
        metadata={
            "name": "loadingAddress",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    delivery_address: Optional[Address] = field(
        default=None,
        metadata={
            "name": "deliveryAddress",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    start_shipping_branch: Optional[int] = field(
        default=None,
        metadata={
            "name": "startShippingBranch",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    complete_shipping_branch: Optional[int] = field(
        default=None,
        metadata={
            "name": "completeShippingBranch",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



import os
import sys
from pathlib import Path

from dotenv import load_dotenv

from mydata import MyDataClient, MyDataClientConfig
from mydata.exceptions import MyDataException, MyDataXMLParseException
from mydata.models import (
    Address,
    Counterpart,
    Country,
    Currency,
    IncomeClassification,
    IncomeClassificationCategory,
    IncomeClassificationValue,
    Invoice,
    InvoiceHeader,
    InvoiceRow,
    InvoiceType,
    Issuer,
    PaymentMethodDetail,
    PaymentMethodType,
    ResponseDoc,
    TaxTotals,
    TaxType,
    VatCategory,
    WithheldPercentCategory,
)

sys.path.append(str(Path(__file__).resolve().parent.parent))
load_dotenv()


def main():
    # Read credentials from environment variables
    user_id = os.getenv("MYDATA_USER", default=None)
    subscription_key = os.getenv("MYDATA_SUBSCRIPTION_KEY", default=None)

    # Configure the client
    config = MyDataClientConfig(environment="sandbox")

    # Initialize the client
    client = MyDataClient(
        user_id=user_id or "your-user-id",
        subscription_key=subscription_key or "your-subscription-key",
        config=config,
    )

    invoice = create_invoice()
    try:
        response_doc = client.send_invoice(invoice=invoice, response_model=ResponseDoc)
        for response in response_doc.response:
            print(
                f"Index: {response.index}  - UUID: {response.invoice_uid} - Mark: {response.invoice_mark} - Status: {response.status_code}"
            )

    except MyDataException as http_err:
        print("HTTP error occurred:", http_err)
    except MyDataXMLParseException as parse_err:
        print("XML parse error occurred:", parse_err)
    except Exception as e:
        print("An unexpected error occurred:", e)


def create_invoice() -> Invoice:
    # Requires a valid VAT number which must be the same as the one of your AADE
    # account if you want canceling/requesting docs to work properly
    issuer = Issuer(vat_number="888888888", country=Country.GR, branch=0)

    a = Address(postal_code="55236", city="Thessaloniki")
    counterpart = Counterpart(
        vat_number="999999999", country=Country.GR, branch=0, address=a
    )

    header = InvoiceHeader()
    header.series = "B"
    header.aa = "12"
    header.issue_date = "2024-12-13"
    header.invoice_type = InvoiceType.VALUE_2_1
    header.currency = Currency.EUR

    payment = PaymentMethodDetail()
    payment.type_value = PaymentMethodType.BANK_ACC_LOCAL
    payment.amount = 1300
    payment.payment_method_info = "Some type of info"

    row = InvoiceRow()
    row.line_number = 1
    row.net_value = 1000
    row.vat_category = VatCategory.VAT_1
    row.vat_amount = 240
    row.income_classification = [
        IncomeClassification(
            classification_type=IncomeClassificationValue.E3_561_001,
            classification_category=IncomeClassificationCategory.CATEGORY1_3,
            amount=1000,
        )
    ]
    tax = TaxTotals()
    tax.tax_type = TaxType.TYPE_1
    tax.tax_category = WithheldPercentCategory.TAX_2
    tax.underlying_value = 1000
    tax.tax_amount = 200

    invoice = Invoice()
    invoice.issuer = issuer
    invoice.counterpart = counterpart
    invoice.invoice_header = header
    invoice.invoice_details = [row]
    invoice.add_payment_method(payment)
    invoice.add_taxes_totals(tax)
    invoice.summarize()

    return invoice


if __name__ == "__main__":
    main()



from dataclasses import dataclass, field
from decimal import Decimal
from typing import Optional

from mydata.models.generated.income_classification_category_type import (
    IncomeClassificationCategory,
)
from mydata.models.generated.income_classification_value_type import (
    IncomeClassificationValue,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/incomeClassificaton/v1.0"


@dataclass
class IncomeClassification:
    """
    Attributes:
        classification_type: Κωδικός Χαρακτηρισμού
        classification_category: Κατηγορία Χαρακτηρισμού
        amount: Ποσό Χαρακτηρισμού
        id: Μοναδικός Αριθμός Χαρακτηρισμού
    """

    class Meta:
        name = "IncomeClassificationType"

    classification_type: Optional[IncomeClassificationValue] = field(
        default=None,
        metadata={
            "name": "classificationType",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        },
    )
    classification_category: Optional[IncomeClassificationCategory] = field(
        default=None,
        metadata={
            "name": "classificationCategory",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            "required": True,
        },
    )
    amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
        },
    )



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.payment_method_type import PaymentMethod

__NAMESPACE__ = "https://www.aade.gr/myDATA/paymentMethod/v1.0"


@dataclass
class PaymentMethodsDoc:
    """
    Μέθοδοι Πληρωμής.
    """

    class Meta:
        namespace = "https://www.aade.gr/myDATA/paymentMethod/v1.0"

    payment_methods: List[PaymentMethod] = field(
        default_factory=list,
        metadata={
            "name": "paymentMethods",
            "type": "Element",
            "min_occurs": 1,
        },
    )



from enum import Enum

__NAMESPACE__ = "https://www.aade.gr/myDATA/incomeClassificaton/v1.0"


class IncomeClassificationCategory(Enum):
    CATEGORY1_1 = "category1_1"
    CATEGORY1_2 = "category1_2"
    CATEGORY1_3 = "category1_3"
    CATEGORY1_4 = "category1_4"
    CATEGORY1_5 = "category1_5"
    CATEGORY1_6 = "category1_6"
    CATEGORY1_7 = "category1_7"
    CATEGORY1_8 = "category1_8"
    CATEGORY1_9 = "category1_9"
    CATEGORY1_10 = "category1_10"
    CATEGORY1_95 = "category1_95"
    CATEGORY3 = "category3"



from dataclasses import dataclass, field
from decimal import Decimal
from typing import Optional

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class TaxTotals:
    """
    Attributes:
        tax_type: Είδος Φόρου
        tax_category: Κατηγορία Φόρου
        underlying_value: Υποκείμενη Αξία
        tax_amount: Ποσό Φόρου
        id:
    """

    class Meta:
        name = "TaxTotalsType"

    tax_type: Optional[int] = field(
        default=None,
        metadata={
            "name": "taxType",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": 1,
            "max_inclusive": 5,
        },
    )
    tax_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "taxCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
        },
    )
    underlying_value: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "underlyingValue",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    tax_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "taxAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from dataclasses import dataclass

from mydata.models.generated.requested_vat_info_type import RequestedVatInfo

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class RequestedVatInfo(RequestedVatInfo):
    class Meta:
        namespace = "http://www.aade.gr/myDATA/invoice/v1.0"



from dataclasses import dataclass, field
from typing import Optional


@dataclass
class InvoiceProvider:
    """
    Attributes:
        issuer_vat: ΑΦΜ Εκδότη
        invoice_provider_mark: Μοναδικός Αριθμός Καταχώρησης
            παραστατικού Παρόχου
        invoice_uid: Αναγνωριστικό οντότητας
        authentication_code: Συμβολοσειρά Αυθεντικοποίησης Παραστατικού
            Παρόχου
    """

    class Meta:
        name = "InvoiceProviderType"

    issuer_vat: Optional[str] = field(
        default=None,
        metadata={
            "name": "issuerVAT",
            "type": "Element",
            "required": True,
        },
    )
    invoice_provider_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceProviderMark",
            "type": "Element",
            "required": True,
        },
    )
    invoice_uid: Optional[str] = field(
        default=None,
        metadata={
            "name": "invoiceUid",
            "type": "Element",
            "required": True,
        },
    )
    authentication_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "authenticationCode",
            "type": "Element",
            "required": True,
        },
    )



from dataclasses import dataclass, field
from typing import Optional

from xsdata.models.datatype import XmlDate

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class Ship:
    """
    Attributes:
        application_id: Αριθμός Δήλωσης διενέργειας δραστηριότητας
        application_date: Ημερομηνία Δήλωσης
        doy:
        ship_id: Στοιχεία Πλοίου
    """

    class Meta:
        name = "ShipType"

    application_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "applicationId",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    application_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "applicationDate",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    doy: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 150,
        },
    )
    ship_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "shipId",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )



from dataclasses import dataclass, field
from decimal import Decimal
from typing import Optional

from mydata.models.generated.expenses_classification_category_type import (
    ExpensesClassificationCategory,
)
from mydata.models.generated.expenses_classification_value_type import (
    ExpensesClassificationValue,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/expensesClassificaton/v1.0"


@dataclass
class ExpensesClassification:
    """
    Attributes:
        classification_type: Κωδικός Χαρακτηρισμού
        classification_category: Κατηγορία Χαρακτηρισμού
        amount: Ποσό Χαρακτηρισμού
        vat_amount: Πόσο Φόρου
        vat_category: Κατηγορία ΦΠΑ
        vat_exemption_category: Κατηγορία Εξαίρεσης ΦΠΑ
        id: Μοναδικός Αριθμός Χαρακτηρισμού
    """

    class Meta:
        name = "ExpensesClassificationType"

    classification_type: Optional[ExpensesClassificationValue] = field(
        default=None,
        metadata={
            "name": "classificationType",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
        },
    )
    classification_category: Optional[ExpensesClassificationCategory] = field(
        default=None,
        metadata={
            "name": "classificationCategory",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
        },
    )
    amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "fraction_digits": 2,
        },
    )
    vat_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "vatAmount",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
            "min_inclusive": Decimal("0"),
            "fraction_digits": 2,
        },
    )
    vat_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "vatCategory",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 8,
        },
    )
    vat_exemption_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "vatExemptionCategory",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 31,
        },
    )
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
        },
    )



"""Configuration module for MyData Client environment and settings."""

from typing import Optional

import requests


class MyDataClientConfig:
    """Configuration class holding environment URLs and default settings for MyData API Client.

    Attributes:
        environment (str): The API environment ('production' or 'sandbox').
        is_provider (bool): Flag indicating whether the client is for provider API endpoints.
        timeout (int): Default timeout for requests in seconds.
        session (requests.Session): The requests session instance used by HTTP client.
    """

    DEV_ERP_URL = "https://mydataapidev.aade.gr/"
    PROD_ERP_URL = "https://mydatapi.aade.gr/myDATA/"

    DEV_PROVIDER_URL = "https://mydataapidev.aade.gr/myDataProvider/"
    PROD_PROVIDER_URL = "https://mydatapi.aade.gr/myDataProvider/"

    def __init__(
        self,
        environment: str = "production",
        is_provider: bool = False,
        timeout: int = 30,
        session: Optional[requests.Session] = None,
    ):
        """Initialize the MyDataClientConfig.

        Args:
            environment (str, optional): 'production' or 'sandbox'. Defaults to 'production'.
            is_provider (bool, optional): Whether to target provider URLs. Defaults to False.
            timeout (int, optional): Request timeout in seconds. Defaults to 30.
            session (Optional[requests.Session], optional): A custom requests session. Defaults to None.
        """
        self.environment = environment.lower()
        self.is_provider = is_provider
        self.timeout = timeout
        self.session = session or requests.Session()

    @property
    def is_sandbox(self) -> bool:
        """Check if the environment is sandbox."""
        return self.environment == "sandbox"

    @property
    def is_production(self) -> bool:
        """Check if the environment is production."""
        return self.environment == "production"

    @property
    def erp_url(self) -> str:
        """The ERP URL based on the selected environment."""
        return self.DEV_ERP_URL if self.is_sandbox else self.PROD_ERP_URL

    @property
    def provider_url(self) -> str:
        """The provider URL based on the selected environment."""
        return self.DEV_PROVIDER_URL if self.is_sandbox else self.PROD_PROVIDER_URL

    @property
    def base_url(self) -> str:
        """Base URL chosen depending on whether the client is a provider or ERP client."""
        return self.provider_url if self.is_provider else self.erp_url



from dataclasses import dataclass, field
from typing import Optional

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class Address:
    """
    Attributes:
        street:
        number: Αριθμός
        postal_code: ΤΚ
        city:
    """

    class Meta:
        name = "AddressType"

    street: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 150,
        },
    )
    number: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    postal_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "postalCode",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    city: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 150,
        },
    )



from mydata.models.generated import (
    AadeBookInvoice,
    Address,
    CancelledInvoice,
    ContinuationTokenType3,
    Country,
    Currency,
    Ecrtoken,
    Entity,
    Error,
    ExpensesClassification,
    ExpensesClassificationCategory,
    ExpensesClassificationsDoc,
    FuelCodes,
    IncomeClassification,
    IncomeClassificationCategory,
    IncomeClassificationValue,
    InvoiceExpensesClassification,
    InvoiceHeader,
    InvoiceIncomeClassification,
    InvoiceProvider,
    InvoiceRow,
    InvoicesDoc,
    InvoicesExpensesClassificationDetail,
    InvoicesIncomeClassificationDetail,
    InvoiceSummary,
    InvoiceType,
    InvoiceVatDetail,
    OtherDeliveryNoteHeader,
    Party,
    PaymentMethod,
    PaymentMethodDetail,
    PaymentMethodsDoc,
    ProviderInfo,
    ProviderSignature,
    ReceptionEmails,
    ReceptionProviders,
    RequestedDoc,
    RequestedProviderDoc,
    RequestedVatInfo,
    Response,
    ResponseDoc,
    Ship,
    TaxTotals,
    TransportDetail,
)

from .counterpart import Counterpart
from .invoice import Invoice
from .issuer import Issuer
from .payment_method_type import PaymentMethodType
from .tax_type import TaxType
from .vat_category import VatCategory
from .withheld_percent_category import WithheldPercentCategory

__all__ = [
    "AadeBookInvoice",
    "Address",
    "CancelledInvoice",
    "ContinuationTokenType3",
    "Country",
    "Currency",
    "Ecrtoken",
    "Entity",
    "Error",
    "ExpensesClassification",
    "ExpensesClassificationCategory",
    "ExpensesClassificationsDoc",
    "FuelCodes",
    "IncomeClassification",
    "IncomeClassificationCategory",
    "IncomeClassificationValue",
    "InvoiceExpensesClassification",
    "InvoiceHeader",
    "InvoiceIncomeClassification",
    "InvoiceProvider",
    "InvoiceRow",
    "InvoicesDoc",
    "InvoicesExpensesClassificationDetail",
    "InvoicesIncomeClassificationDetail",
    "InvoiceSummary",
    "InvoiceType",
    "InvoiceVatDetail",
    "OtherDeliveryNoteHeader",
    "Party",
    "PaymentMethod",
    "PaymentMethodDetail",
    "PaymentMethodsDoc",
    "ProviderInfo",
    "ProviderSignature",
    "ReceptionEmails",
    "ReceptionProviders",
    "RequestedDoc",
    "RequestedProviderDoc",
    "RequestedVatInfo",
    "Response",
    "ResponseDoc",
    "Ship",
    "TaxTotals",
    "TransportDetail",
]

__all__ += [
    "Counterpart",
    "Issuer",
    "Invoice",
    "PaymentMethodType",
    "TaxType",
    "VatCategory",
    "WithheldPercentCategory",
]



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.error_type import Error
from mydata.models.generated.reception_emails_type import ReceptionEmails
from mydata.models.generated.reception_providers_type import ReceptionProviders


@dataclass
class Response:
    """
    Attributes:
        index: ΑΑ γραμμής οντότητας
        invoice_uid: Αναγνωριστικό οντότητας
        invoice_mark: Μοναδικός Αριθμός Καταχώρησης παραστατικού
        qr_url: QR Code Url
        classification_mark: Μοναδικός Αριθμός Παραλαβής Χαρακτηρισμού
        cancellation_mark: Μοναδικός Αριθμός Ακύρωσης
        payment_method_mark: Μοναδικός Αριθμός Παραλαβής Τρόπου Πληρωμής
        authentication_code: Συμβολοσειρά Αυθεντικοποίησης Παρόχου
        reception_providers: Πάροχοι Λήπτη
        reception_emails: Email Παραλαβής
        errors: Λίστα Σφαλμάτων
        status_code: Κωδικός αποτελέσματος
    """

    class Meta:
        name = "ResponseType"

    index: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
        },
    )
    invoice_uid: Optional[str] = field(
        default=None,
        metadata={
            "name": "invoiceUid",
            "type": "Element",
        },
    )
    invoice_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceMark",
            "type": "Element",
        },
    )
    qr_url: Optional[str] = field(
        default=None,
        metadata={
            "name": "qrUrl",
            "type": "Element",
        },
    )
    classification_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "classificationMark",
            "type": "Element",
        },
    )
    cancellation_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "cancellationMark",
            "type": "Element",
        },
    )
    payment_method_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "paymentMethodMark",
            "type": "Element",
        },
    )
    authentication_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "authenticationCode",
            "type": "Element",
        },
    )
    reception_providers: Optional[ReceptionProviders] = field(
        default=None,
        metadata={
            "name": "receptionProviders",
            "type": "Element",
        },
    )
    reception_emails: Optional[ReceptionEmails] = field(
        default=None,
        metadata={
            "name": "receptionEmails",
            "type": "Element",
        },
    )
    errors: Optional["Response.Errors"] = field(
        default=None,
        metadata={
            "type": "Element",
        },
    )
    status_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "statusCode",
            "type": "Element",
            "required": True,
        },
    )

    @dataclass
    class Errors:
        error: List[Error] = field(
            default_factory=list,
            metadata={
                "type": "Element",
                "min_occurs": 1,
            },
        )



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.provider_info_type import ProviderInfo


@dataclass
class ReceptionProviders:
    """
    Attributes:
        provider_info: Πληροφορίες Παρόχου
    """

    class Meta:
        name = "receptionProvidersType"

    provider_info: List[ProviderInfo] = field(
        default_factory=list,
        metadata={
            "name": "ProviderInfo",
            "type": "Element",
        },
    )



from dataclasses import dataclass, field
from decimal import Decimal
from typing import Optional

from mydata.models.generated.ecrtoken_type import Ecrtoken
from mydata.models.generated.provider_signature_type import ProviderSignature

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class PaymentMethodDetail:
    """
    Attributes:
        type_value: Τύπος Πληρωμής
        amount: Αναλογούν Ποσό
        payment_method_info: Λοιπές Πληροφορίες
        tip_amount: Φιλοδώρημα
        transaction_id: Μοναδική Ταυτότητα Πληρωμής
        tid: tid POS
        providers_signature: Υπογραφή Πληρωμής Παρόχου
        ecrtoken: Υπογραφή Πληρωμής ΦΗΜ με σύστημα λογισμικού (ERP)
    """

    class Meta:
        name = "PaymentMethodDetailType"

    type_value: Optional[int] = field(
        default=None,
        metadata={
            "name": "type",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": 1,
            "max_inclusive": 8,
        },
    )
    amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    payment_method_info: Optional[str] = field(
        default=None,
        metadata={
            "name": "paymentMethodInfo",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    tip_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "tipAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    transaction_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "transactionId",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    tid: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 200,
        },
    )
    providers_signature: Optional[ProviderSignature] = field(
        default=None,
        metadata={
            "name": "ProvidersSignature",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    ecrtoken: Optional[Ecrtoken] = field(
        default=None,
        metadata={
            "name": "ECRToken",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from dataclasses import dataclass, field
from typing import Optional


@dataclass
class Error:
    """
    Attributes:
        message: Μήνυμα Σφάλματος
        code: Κωδικός Σφάλαματος
    """

    class Meta:
        name = "ErrorType"

    message: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "required": True,
        },
    )
    code: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "required": True,
        },
    )



import os
import sys
from pathlib import Path

from dotenv import load_dotenv

from mydata import MyDataClient, MyDataClientConfig
from mydata.exceptions import MyDataException, MyDataXMLParseException

sys.path.append(str(Path(__file__).resolve().parent.parent))
load_dotenv()


def main():
    # Read credentials from environment variables
    user_id = os.getenv("MYDATA_USER", default=None)
    subscription_key = os.getenv("MYDATA_SUBSCRIPTION_KEY", default=None)

    # Configure the client
    config = MyDataClientConfig(environment="sandbox")

    # Initialize the client
    client = MyDataClient(
        user_id=user_id or "your-user-id",
        subscription_key=subscription_key or "your-subscription-key",
        config=config,
    )

    try:
        response_doc = client.request_transmitted_docs()
        print(response_doc)

    except MyDataException as http_err:
        print("HTTP error occurred:", http_err)
    except MyDataXMLParseException as parse_err:
        print("XML parse error occurred:", parse_err)
    except Exception as e:
        print("An unexpected error occurred:", e)


if __name__ == "__main__":
    main()



from enum import Enum


class TaxType(int, Enum):
    # Παρακρατούμενος Φόρος
    TYPE_1 = 1
    # Τέλη
    TYPE_2 = 2
    # Λοιποί Φόροι
    TYPE_3 = 3
    # Χαρτόσημο
    TYPE_4 = 4
    # Κρατήσεις
    TYPE_5 = 5



from decimal import Decimal
from typing import Dict, List, Optional, Tuple

from mydata.models.generated.expenses_classification_category_type import (
    ExpensesClassificationCategory,
)
from mydata.models.generated.expenses_classification_type import ExpensesClassification
from mydata.models.generated.expenses_classification_value_type import (
    ExpensesClassificationValue,
)
from mydata.models.generated.income_classification_category_type import (
    IncomeClassificationCategory,
)
from mydata.models.generated.income_classification_type import IncomeClassification
from mydata.models.generated.income_classification_value_type import (
    IncomeClassificationValue,
)
from mydata.models.generated.invoice_row_type import InvoiceRow
from mydata.models.vat_category import VatCategory
from mydata.models.vat_exemption import VatExemption


class ClassificationsGrouper:
    def __init__(self, rows=Optional[List[InvoiceRow]]):
        self.rows = rows
        self.income_classifications: Dict[str, Dict[str, Decimal]] = {}
        self.expenses_classifications: Dict[
            str, Dict[str, Dict[str, Dict[str, Dict[str, Decimal]]]]
        ] = {}

    def create_groups(
        self, options: Dict = None
    ) -> Tuple[List[IncomeClassification], List[ExpensesClassification]]:
        if not self.rows:
            return [], []

        for row in self.rows:
            self._group_income_classifications(row)
            self._group_expenses_classifications(row)

        return (
            self._flatten_income_classifications(options),
            self._flatten_expenses_classifications(options),
        )

    def _flatten_income_classifications(
        self, options: Dict = None
    ) -> List[IncomeClassification]:
        flattened_income_classifications = []

        for category_key, types in self.income_classifications.items():
            category = IncomeClassificationCategory(category_key)

            for type_key, amount in types.items():
                classification_type = (
                    IncomeClassificationValue(type_key) if type_key else None
                )

                icls = IncomeClassification(
                    classification_category=category,
                    classification_type=classification_type,
                    amount=round(amount, 2),
                )

                if options and options.get("enableClassificationIds"):
                    icls.id = len(flattened_income_classifications) + 1

                flattened_income_classifications.append(icls)

        return flattened_income_classifications

    def _flatten_expenses_classifications(
        self, options: Dict = None
    ) -> List[ExpensesClassification]:
        flattened_expenses_classifications = []

        for category_key, types in self.expenses_classifications.items():
            category = ExpensesClassificationCategory(category_key)

            for type_key, vat_categories in types.items():
                classification_type = (
                    ExpensesClassificationValue(type_key) if type_key else None
                )

                for vat_category_key, exemptions in vat_categories.items():
                    vat_category = (
                        VatCategory(vat_category_key) if vat_category_key else None
                    )

                    for exemption_key, amounts in exemptions.items():
                        vat_exemption = (
                            VatExemption(exemption_key) if exemption_key else None
                        )

                        ecls = ExpensesClassification(
                            classification_category=category,
                            classification_type=classification_type,
                            vat_category=vat_category,
                            vat_exemption_category=vat_exemption,
                            amount=round(amounts["amount"], 2),
                            vat_amount=round(amounts["vatAmount"], 2),
                        )

                        if options and options.get("enableClassificationIds"):
                            ecls.id = len(flattened_expenses_classifications) + 1

                        flattened_expenses_classifications.append(ecls)

        return flattened_expenses_classifications

    def _group_income_classifications(self, row: InvoiceRow) -> None:
        if not row.income_classification:
            return

        for icls in row.income_classification:
            category_key = icls.classification_category.value or ""
            type_key = icls.classification_type.value or ""

            previous_amount = self.income_classifications.get(category_key, {}).get(
                type_key, Decimal(0)
            )
            new_amount = previous_amount + abs(icls.amount or Decimal(0))

            self.income_classifications.setdefault(category_key, {})[type_key] = (
                new_amount
            )

    def _group_expenses_classifications(self, row: InvoiceRow) -> None:
        if not row.expenses_classification:
            return

        for ecls in row.expenses_classification:
            category_key = ecls.classification_category.value or ""
            type_key = ecls.classification_type.value or ""
            vat_category_key = ecls.vat_category.value or ""
            vat_exemption_key = ecls.vat_exemption_category.value or ""

            previous_amount = self._get_summarized_expenses_classification(
                ecls, "amount"
            )
            previous_vat_amount = self._get_summarized_expenses_classification(
                ecls, "vatAmount"
            )

            new_amount = previous_amount + abs(ecls.amount or Decimal(0))
            new_vat_amount = previous_vat_amount + abs(ecls.vat_amount or Decimal(0))

            self.expenses_classifications.setdefault(category_key, {}).setdefault(
                type_key, {}
            ).setdefault(vat_category_key, {})[vat_exemption_key] = {
                "amount": new_amount,
                "vatAmount": new_vat_amount,
            }

    def _get_summarized_expenses_classification(
        self, ecls: ExpensesClassification, key: str
    ) -> Decimal:
        category_key = ecls.classification_category.value or ""
        type_key = ecls.classification_type.value or ""
        vat_category_key = ecls.vat_category.value or ""
        vat_exemption_key = ecls.vat_exemption_category.value or ""

        return (
            self.expenses_classifications.get(category_key, {})
            .get(type_key, {})
            .get(vat_category_key, {})
            .get(vat_exemption_key, {})
            .get(key, Decimal(0))
        )



from dataclasses import dataclass, field
from decimal import Decimal
from typing import List, Optional

from mydata.models.generated.expenses_classification_type import (
    ExpensesClassification,
)
from mydata.models.generated.fuel_codes import FuelCodes
from mydata.models.generated.income_classification_type import (
    IncomeClassification,
)
from mydata.models.generated.ship_type import Ship

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class InvoiceRow:
    """
    Attributes:
        line_number: ΑΑ Γραμμής
        rec_type: Είδος Γραμμής
        taric_no: Κωδικός Taric
        item_code: Κωδικός Είδους
        item_descr: Περιγραφή Είδους
        fuel_code: Κωδικός Καυσίμου
        quantity: Ποσότητα
        measurement_unit: Είδος Ποσότητας
        invoice_detail_type: Επισήμανση
        net_value: Καθαρή Αξία
        vat_category: Κατηγορία ΦΠΑ
        vat_amount: Ποσό ΦΠΑ
        vat_exemption_category: Κατηγορία Αιτίας Εξαίρεσης ΦΠΑ
        dienergia: ΠΟΛ 1177/2018 Αρ. 27
        discount_option: Δικαίωμα Έκπτωσης
        withheld_amount: Ποσό Παρ. Φόρου
        withheld_percent_category: Κατηγορία Συντελεστή  Παρ. Φόρου
        stamp_duty_amount: Ποσό Χαρτοσήμου
        stamp_duty_percent_category: Κατηγορία Συντελεστή  Χαρτοσήμου
        fees_amount: Ποσό Τελών
        fees_percent_category: Κατηγορία Συντελεστή Τελών
        other_taxes_percent_category: Κατηγορία Συντελεστή Λοιπών Φόρων
        other_taxes_amount: Ποσό Φόρου Διαμονης
        deductions_amount: Ποσό Κρατήσεων
        line_comments: Σχόλια Γραμμής
        income_classification: Λίστα Χαρακτηρισμών Εσόδων
        expenses_classification: Λίστα Χαρακτηρισμού Εξόδων
        quantity15: Ποσότητα Θερμοκρασίας 15 βαθμών
        other_measurement_unit_quantity: Πλήθος Μονάδας Μέτρησης Τεμάχια
            Άλλα
        other_measurement_unit_title: Τίτλος Μονάδας Μέτρησης Τεμάχια
            Άλλα
        not_vat195: Ένδειξη μη συμμετοχής στο ΦΠΑ (έσοδα – εκροές)
    """

    class Meta:
        name = "InvoiceRowType"

    line_number: Optional[int] = field(
        default=None,
        metadata={
            "name": "lineNumber",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": 1,
        },
    )
    rec_type: Optional[int] = field(
        default=None,
        metadata={
            "name": "recType",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 7,
        },
    )
    taric_no: Optional[str] = field(
        default=None,
        metadata={
            "name": "TaricNo",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "length": 10,
        },
    )
    item_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "itemCode",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 50,
        },
    )
    item_descr: Optional[str] = field(
        default=None,
        metadata={
            "name": "itemDescr",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 300,
        },
    )
    fuel_code: Optional[FuelCodes] = field(
        default=None,
        metadata={
            "name": "fuelCode",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    quantity: Optional[Decimal] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_exclusive": Decimal("0"),
        },
    )
    measurement_unit: Optional[int] = field(
        default=None,
        metadata={
            "name": "measurementUnit",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 7,
        },
    )
    invoice_detail_type: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceDetailType",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 2,
        },
    )
    net_value: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "netValue",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    vat_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "vatCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": 1,
            "max_inclusive": 10,
        },
    )
    vat_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "vatAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    vat_exemption_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "vatExemptionCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 31,
        },
    )
    dienergia: Optional[Ship] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    discount_option: Optional[bool] = field(
        default=None,
        metadata={
            "name": "discountOption",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    withheld_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "withheldAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    withheld_percent_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "withheldPercentCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 18,
        },
    )
    stamp_duty_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "stampDutyAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    stamp_duty_percent_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "stampDutyPercentCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 4,
        },
    )
    fees_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "feesAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    fees_percent_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "feesPercentCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 22,
        },
    )
    other_taxes_percent_category: Optional[int] = field(
        default=None,
        metadata={
            "name": "otherTaxesPercentCategory",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": 1,
            "max_inclusive": 30,
        },
    )
    other_taxes_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "otherTaxesAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    deductions_amount: Optional[Decimal] = field(
        default=None,
        metadata={
            "name": "deductionsAmount",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_inclusive": Decimal("0"),
            "total_digits": 15,
            "fraction_digits": 2,
        },
    )
    line_comments: Optional[str] = field(
        default=None,
        metadata={
            "name": "lineComments",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 150,
        },
    )
    income_classification: List[IncomeClassification] = field(
        default_factory=list,
        metadata={
            "name": "incomeClassification",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    expenses_classification: List[ExpensesClassification] = field(
        default_factory=list,
        metadata={
            "name": "expensesClassification",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    quantity15: Optional[Decimal] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "min_exclusive": Decimal("0"),
        },
    )
    other_measurement_unit_quantity: Optional[int] = field(
        default=None,
        metadata={
            "name": "otherMeasurementUnitQuantity",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    other_measurement_unit_title: Optional[str] = field(
        default=None,
        metadata={
            "name": "otherMeasurementUnitTitle",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "max_length": 150,
        },
    )
    not_vat195: Optional[bool] = field(
        default=None,
        metadata={
            "name": "notVAT195",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from dataclasses import dataclass, field
from typing import Optional

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class ContinuationTokenType1:
    class Meta:
        name = "ContinuationTokenType"

    next_partition_key: Optional[str] = field(
        default=None,
        metadata={
            "name": "nextPartitionKey",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    next_row_key: Optional[str] = field(
        default=None,
        metadata={
            "name": "nextRowKey",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from enum import Enum


class PaymentMethodType(Enum):
    BANK_ACC_LOCAL = 1
    BANK_ACC_FOR = 2
    CASH = 3
    CHEQUE = 4
    CREDIT = 5
    WEB_BANKING = 6
    POS = 7
    IRIS = 8

    def __str__(self) -> str:
        labels = {
            PaymentMethodType.BANK_ACC_LOCAL: "Επαγ. Λογαριασμός Πληρωμών Ημεδαπής",
            PaymentMethodType.BANK_ACC_FOR: "Επαγ. Λογαριασμός Πληρωμών Αλλοδαπής",
            PaymentMethodType.CASH: "Μετρητά",
            PaymentMethodType.CHEQUE: "Επιταγή",
            PaymentMethodType.CREDIT: "Επί Πιστώσει",
            PaymentMethodType.WEB_BANKING: "Web Banking",
            PaymentMethodType.POS: "POS / e-POS",
            PaymentMethodType.IRIS: "Άμεσες Πληρωμές IRIS",
        }
        return labels.get(self, "Unknown Payment method type")



"""Custom exception classes for MyData client operations."""


class MyDataException(Exception):
    """Base exception for all MyData-related errors."""

    pass


class MyDataHTTPException(MyDataException):
    """Base exception for HTTP-related errors when communicating with the MyData API."""

    def __init__(self, status_code: int, url: str, message: str = ""):
        self.status_code = status_code
        self.url = url
        self.message = message
        super().__init__(f"HTTP[{status_code}] {url}: {message}")


class MyDataXMLParseException(MyDataException):
    """Exception raised for errors encountered during XML parsing."""

    def __init__(self, original_exception: Exception):
        self.original_exception = original_exception
        super().__init__(f"Failed to parse XML data: {original_exception}")


class MyDataInvalidResponseException(MyDataHTTPException):
    """Exception raised when the response from the server is empty or invalid."""

    def __init__(
        self,
        url: str,
        message: str = "Empty or invalid response received from AADE MyData API",
    ):
        super().__init__(status_code=200, url=url, message=message)


class MyDataAuthenticationException(MyDataHTTPException):
    """Exception raised for authentication failures (e.g., HTTP 401)."""

    pass


class MyDataConnectionException(MyDataHTTPException):
    """Exception raised for connection-related errors (HTTP code 0 or similar)."""

    pass


class MyDataRateLimitExceededException(MyDataHTTPException):
    """Exception raised when rate limit is exceeded (HTTP 429)."""

    pass


class MyDataDataException(MyDataHTTPException):
    """Generic data or transmission error exception for unexpected HTTP errors."""

    pass



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.continuation_token_type_1 import (
    ContinuationTokenType1,
)
from mydata.models.generated.invoice_vat_detail_type import InvoiceVatDetail

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class RequestedVatInfo:
    class Meta:
        name = "RequestedVatInfoType"

    continuation_token: Optional[ContinuationTokenType1] = field(
        default=None,
        metadata={
            "name": "continuationToken",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )
    vat_info: List[InvoiceVatDetail] = field(
        default_factory=list,
        metadata={
            "name": "VatInfo",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
        },
    )



from dataclasses import dataclass, field
from typing import List

from mydata.models.generated.response_type import Response


@dataclass
class ResponseDoc:
    """
    Comment describing your root element.
    """

    response: List[Response] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "min_occurs": 1,
        },
    )



from dataclasses import dataclass, field
from typing import Optional

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class TransportDetail:
    """
    Attributes:
        vehicle_number: Αριθμός Μεταφορικού Μέσου
    """

    class Meta:
        name = "TransportDetailType"

    vehicle_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "vehicleNumber",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
            "max_length": 50,
        },
    )



from enum import Enum

__NAMESPACE__ = "https://www.aade.gr/myDATA/expensesClassificaton/v1.0"


class ExpensesClassificationCategory(Enum):
    CATEGORY2_1 = "category2_1"
    CATEGORY2_2 = "category2_2"
    CATEGORY2_3 = "category2_3"
    CATEGORY2_4 = "category2_4"
    CATEGORY2_5 = "category2_5"
    CATEGORY2_6 = "category2_6"
    CATEGORY2_7 = "category2_7"
    CATEGORY2_8 = "category2_8"
    CATEGORY2_9 = "category2_9"
    CATEGORY2_10 = "category2_10"
    CATEGORY2_11 = "category2_11"
    CATEGORY2_12 = "category2_12"
    CATEGORY2_13 = "category2_13"
    CATEGORY2_14 = "category2_14"
    CATEGORY2_95 = "category2_95"



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.payment_method_detail_type import (
    PaymentMethodDetail,
)

__NAMESPACE__ = "https://www.aade.gr/myDATA/paymentMethod/v1.0"


@dataclass
class PaymentMethod:
    """
    Attributes:
        invoice_mark: Μοναδικός Αριθμός Καταχώρησης Παραστατικού
        payment_method_mark: Αποδεικτικό Λήψης Τρόπων Πληρωμής.
            Συμπληρώνεται από την Υπηρεσία
        entity_vat_number: ΑΦΜ Οντότητας Αναφοράς
        payment_method_details:
    """

    class Meta:
        name = "PaymentMethodType"

    invoice_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceMark",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/paymentMethod/v1.0",
            "required": True,
        },
    )
    payment_method_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "paymentMethodMark",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/paymentMethod/v1.0",
        },
    )
    entity_vat_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "entityVatNumber",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/paymentMethod/v1.0",
        },
    )
    payment_method_details: List[PaymentMethodDetail] = field(
        default_factory=list,
        metadata={
            "name": "paymentMethodDetails",
            "type": "Element",
            "namespace": "https://www.aade.gr/myDATA/paymentMethod/v1.0",
            "min_occurs": 1,
        },
    )



from typing import Optional

from xsdata.formats.dataclass.models.elements import XmlType

from mydata.utils.invoice_summarizer import InvoiceSummarizer

from .counterpart import Counterpart
from .generated.aade_book_invoice_type import AadeBookInvoice
from .generated.invoice_summary_type import InvoiceSummary
from .generated.payment_method_detail_type import PaymentMethodDetail
from .generated.tax_totals_type import TaxTotals
from .issuer import Issuer


class Invoice(AadeBookInvoice):
    issuer: Optional[Issuer]
    counterpart: Optional[Counterpart]

    class Meta:
        name = "invoice"
        nillable = False
        namespace = "http://www.aade.gr/myDATA/invoice/v1.0"
        type = XmlType.ELEMENT

    def add_payment_method(self, method: PaymentMethodDetail):
        if not self.payment_methods:
            self.payment_methods = AadeBookInvoice.PaymentMethods()
        self.payment_methods.payment_method_details.append(method)

    def add_taxes_totals(self, tax: TaxTotals):
        if not self.taxes_totals:
            self.taxes_totals = AadeBookInvoice.TaxesTotals()
        self.taxes_totals.taxes.append(tax)

    def summarize(self) -> InvoiceSummary:
        s = InvoiceSummarizer(self).summarize()
        self.invoice_summary = s
        return s



from dataclasses import dataclass, field
from typing import Optional

from xsdata.models.datatype import XmlDate

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class CancelledInvoice:
    """
    Attributes:
        invoice_mark: Μοναδικός Αριθμός Καταχώρησης του ακυρωμένου
            Παραστατικού
        cancellation_mark: Μοναδικός Αριθμός Καταχώρησης της Ακύρωσης
        cancellation_date: Ημερομηνία Ακύρωσης Παραστατικού
    """

    class Meta:
        name = "CancelledInvoiceType"

    invoice_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "invoiceMark",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    cancellation_mark: Optional[int] = field(
        default=None,
        metadata={
            "name": "cancellationMark",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )
    cancellation_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "cancellationDate",
            "type": "Element",
            "namespace": "http://www.aade.gr/myDATA/invoice/v1.0",
            "required": True,
        },
    )



from dataclasses import dataclass, field
from typing import List


@dataclass
class ReceptionEmails:
    """
    Attributes:
        email: Email
    """

    class Meta:
        name = "receptionEmailsType"

    email: List[str] = field(
        default_factory=list,
        metadata={
            "type": "Element",
        },
    )



from dataclasses import dataclass, field
from typing import List, Optional

from mydata.models.generated.aade_book_invoice_type import AadeBookInvoice
from mydata.models.generated.cancelled_invoice_type import CancelledInvoice
from mydata.models.generated.continuation_token_type_3 import (
    ContinuationTokenType3,
)
from mydata.models.generated.invoice_expenses_classification_type import (
    InvoiceExpensesClassification,
)
from mydata.models.generated.invoice_income_classification_type import (
    InvoiceIncomeClassification,
)
from mydata.models.generated.payment_method_type import PaymentMethod

__NAMESPACE__ = "http://www.aade.gr/myDATA/invoice/v1.0"


@dataclass
class RequestedDoc:
    class Meta:
        namespace = "http://www.aade.gr/myDATA/invoice/v1.0"

    continuation_token: Optional[ContinuationTokenType3] = field(
        default=None,
        metadata={
            "name": "continuationToken",
            "type": "Element",
        },
    )
    invoices_doc: Optional["RequestedDoc.InvoicesDoc"] = field(
        default=None,
        metadata={
            "name": "invoicesDoc",
            "type": "Element",
        },
    )
    cancelled_invoices_doc: Optional["RequestedDoc.CancelledInvoicesDoc"] = field(
        default=None,
        metadata={
            "name": "cancelledInvoicesDoc",
            "type": "Element",
        },
    )
    income_classifications_doc: Optional["RequestedDoc.IncomeClassificationsDoc"] = (
        field(
            default=None,
            metadata={
                "name": "incomeClassificationsDoc",
                "type": "Element",
            },
        )
    )
    expenses_classifications_doc: Optional[
        "RequestedDoc.ExpensesClassificationsDoc"
    ] = field(
        default=None,
        metadata={
            "name": "expensesClassificationsDoc",
            "type": "Element",
        },
    )
    payment_methods_doc: Optional["RequestedDoc.PaymentMethodsDoc"] = field(
        default=None,
        metadata={
            "name": "paymentMethodsDoc",
            "type": "Element",
        },
    )

    @dataclass
    class InvoicesDoc:
        invoice: List[AadeBookInvoice] = field(
            default_factory=list,
            metadata={
                "type": "Element",
            },
        )

    @dataclass
    class CancelledInvoicesDoc:
        cancelled_invoice: List[CancelledInvoice] = field(
            default_factory=list,
            metadata={
                "name": "cancelledInvoice",
                "type": "Element",
            },
        )

    @dataclass
    class IncomeClassificationsDoc:
        income_invoice_classification: List[InvoiceIncomeClassification] = field(
            default_factory=list,
            metadata={
                "name": "incomeInvoiceClassification",
                "type": "Element",
                "namespace": "https://www.aade.gr/myDATA/incomeClassificaton/v1.0",
            },
        )

    @dataclass
    class ExpensesClassificationsDoc:
        expenses_invoice_classification: List[InvoiceExpensesClassification] = field(
            default_factory=list,
            metadata={
                "name": "expensesInvoiceClassification",
                "type": "Element",
                "namespace": "https://www.aade.gr/myDATA/expensesClassificaton/v1.0",
            },
        )

    @dataclass
    class PaymentMethodsDoc:
        payment_methods: List[PaymentMethod] = field(
            default_factory=list,
            metadata={
                "name": "paymentMethods",
                "type": "Element",
            },
        )



from mydata.models.generated.aade_book_invoice_type import AadeBookInvoice
from mydata.models.generated.address_type import Address
from mydata.models.generated.cancelled_invoice_type import CancelledInvoice
from mydata.models.generated.continuation_token_type_1 import (
    ContinuationTokenType1,
)
from mydata.models.generated.continuation_token_type_2 import (
    ContinuationTokenType2,
)
from mydata.models.generated.continuation_token_type_3 import (
    ContinuationTokenType3,
)
from mydata.models.generated.country_type import Country
from mydata.models.generated.currency_type import Currency
from mydata.models.generated.ecrtoken_type import Ecrtoken
from mydata.models.generated.entity_type import Entity
from mydata.models.generated.error_type import Error
from mydata.models.generated.expenses_classification_category_type import (
    ExpensesClassificationCategory,
)
from mydata.models.generated.expenses_classification_type import (
    ExpensesClassification,
)
from mydata.models.generated.expenses_classification_value_type import (
    ExpensesClassificationValue,
)
from mydata.models.generated.expenses_classifications_doc import (
    ExpensesClassificationsDoc,
)
from mydata.models.generated.fuel_codes import FuelCodes
from mydata.models.generated.income_classification_category_type import (
    IncomeClassificationCategory,
)
from mydata.models.generated.income_classification_type import (
    IncomeClassification,
)
from mydata.models.generated.income_classification_value_type import (
    IncomeClassificationValue,
)
from mydata.models.generated.income_classifications_doc import (
    IncomeClassificationsDoc,
)
from mydata.models.generated.invoice_expenses_classification_type import (
    InvoiceExpensesClassification,
)
from mydata.models.generated.invoice_header_type import InvoiceHeader
from mydata.models.generated.invoice_income_classification_type import (
    InvoiceIncomeClassification,
)
from mydata.models.generated.invoice_provider_type import InvoiceProvider
from mydata.models.generated.invoice_row_type import InvoiceRow
from mydata.models.generated.invoice_summary_type import InvoiceSummary
from mydata.models.generated.invoice_type import InvoiceType
from mydata.models.generated.invoice_vat_detail_type import InvoiceVatDetail
from mydata.models.generated.invoices_doc import InvoicesDoc
from mydata.models.generated.invoices_expenses_classification_detail_type import (
    InvoicesExpensesClassificationDetail,
)
from mydata.models.generated.invoices_income_classification_detail_type import (
    InvoicesIncomeClassificationDetail,
)
from mydata.models.generated.other_delivery_note_header_type import (
    OtherDeliveryNoteHeader,
)
from mydata.models.generated.party_type import Party
from mydata.models.generated.payment_method_detail_type import (
    PaymentMethodDetail,
)
from mydata.models.generated.payment_method_type import PaymentMethod
from mydata.models.generated.payment_methods_doc import PaymentMethodsDoc
from mydata.models.generated.provider_info_type import ProviderInfo
from mydata.models.generated.provider_signature_type import ProviderSignature
from mydata.models.generated.reception_emails_type import ReceptionEmails
from mydata.models.generated.reception_providers_type import ReceptionProviders
from mydata.models.generated.requested_doc import RequestedDoc
from mydata.models.generated.requested_provider_doc import RequestedProviderDoc
from mydata.models.generated.requested_vat_info import RequestedVatInfo
from mydata.models.generated.requested_vat_info_type import RequestedVatInfo
from mydata.models.generated.response_doc import ResponseDoc
from mydata.models.generated.response_type import Response
from mydata.models.generated.ship_type import Ship
from mydata.models.generated.tax_totals_type import TaxTotals
from mydata.models.generated.transport_detail_type import TransportDetail

__all__ = [
    "AadeBookInvoice",
    "Address",
    "CancelledInvoice",
    "ContinuationTokenType1",
    "ContinuationTokenType2",
    "ContinuationTokenType3",
    "Country",
    "Currency",
    "Ecrtoken",
    "Entity",
    "Error",
    "ExpensesClassificationCategory",
    "ExpensesClassification",
    "ExpensesClassificationValue",
    "ExpensesClassificationsDoc",
    "FuelCodes",
    "IncomeClassificationCategory",
    "IncomeClassification",
    "IncomeClassificationValue",
    "IncomeClassificationsDoc",
    "InvoiceExpensesClassification",
    "InvoiceHeader",
    "InvoiceIncomeClassification",
    "InvoiceProvider",
    "InvoiceRow",
    "InvoiceSummary",
    "InvoiceType",
    "InvoiceVatDetail",
    "InvoicesDoc",
    "InvoicesExpensesClassificationDetail",
    "InvoicesIncomeClassificationDetail",
    "OtherDeliveryNoteHeader",
    "Party",
    "PaymentMethodDetail",
    "PaymentMethod",
    "PaymentMethodsDoc",
    "ProviderInfo",
    "ProviderSignature",
    "ReceptionEmails",
    "ReceptionProviders",
    "RequestedDoc",
    "RequestedProviderDoc",
    "RequestedVatInfo",
    "RequestedVatInfo",
    "ResponseDoc",
    "Response",
    "Ship",
    "TaxTotals",
    "TransportDetail",
]
